@@ -196,10 +196,14 @@ main (gint argc, gchar **argv)
             /* Get the dialog widget */
             dialog = gtk_builder_get_object (builder, "dialog");
 
-            /* run the dialog */
-            gtk_dialog_run (GTK_DIALOG (dialog));
+            gtk_widget_show (GTK_WIDGET (dialog));
+            g_signal_connect (dialog, "response", G_CALLBACK (gtk_main_quit), NULL);
+
+            /* To prevent the settings dialog to be saved in the session */
+            gdk_set_sm_client_id ("FAKE ID");
+
+            gtk_main ();
 
-            /* destroy the dialog */
             gtk_widget_destroy (GTK_WIDGET (dialog));
         }
         else
@@ -217,6 +221,9 @@ main (gint argc, gchar **argv)
             gtk_widget_reparent (GTK_WIDGET (plug_child), plug);
             gtk_widget_show (GTK_WIDGET (plug_child));
 
+            /* To prevent the settings dialog to be saved in the session */
+            gdk_set_sm_client_id ("FAKE ID");
+
             /* Enter main loop */
             gtk_main ();
         }
