@@ -42,6 +42,10 @@
 
 #define MAX_DENOMINATOR (100.00)
 
+/* Xi 1.4 is required */
+#define MIN_XI_VERS_MAJOR 1
+#define MIN_XI_VERS_MINOR 4
+
 /* test if the required version of inputproto (1.4.2) is available */
 #if XI_Add_DevicePresenceNotify_Major >= 1 && defined (DeviceRemoved)
 #define HAS_DEVICE_HOTPLUGGING
@@ -54,9 +58,6 @@
 
 
 
-static void             xfce_pointers_helper_class_init                     (XfcePointersHelperClass *klass);
-static void             xfce_pointers_helper_init                           (XfcePointersHelper      *helper);
-static void             xfce_pointers_helper_finalize                       (GObject                 *object);
 static void             xfce_pointers_helper_change_button_mapping_swap     (guchar                  *buttonmap,
                                                                              gshort                   num_buttons,
                                                                              gint                     id_1,
@@ -114,10 +115,7 @@ G_DEFINE_TYPE (XfcePointersHelper, xfce_pointers_helper, G_TYPE_OBJECT);
 static void
 xfce_pointers_helper_class_init (XfcePointersHelperClass *klass)
 {
-    GObjectClass *gobject_class;
 
-    gobject_class = G_OBJECT_CLASS (klass);
-    gobject_class->finalize = xfce_pointers_helper_finalize;
 }
 
 
@@ -133,23 +131,27 @@ xfce_pointers_helper_init (XfcePointersHelper *helper)
 
     /* get the default display */
     xdisplay = gdk_x11_display_get_xdisplay (gdk_display_get_default ());
-    
+
     /* query the extension version */
     version = XGetExtensionVersion (xdisplay, INAME);
-    
-    /* check for Xi 1.4 */
-    if (!version || !version->present || version->major_version < 1 || version->minor_version < 4)
-    {
-        /* print error */
-        g_critical ("XI is not present or too old.");
 
-        /* no channel */
-        helper->channel = NULL;
+    /* check for Xi */
+    if (version == NULL || !version->present)
+    {
+        g_critical ("XI is not present.");
+    }
+    else if (version->major_version < MIN_XI_VERS_MAJOR
+             || (version->major_version == MIN_XI_VERS_MAJOR
+                 && version->minor_version < MIN_XI_VERS_MINOR))
+    {
+        g_critical ("Your XI is too old (%d.%d) version %d.%d is required.",
+                    version->major_version, version->minor_version,
+                    MIN_XI_VERS_MAJOR, MIN_XI_VERS_MINOR);
     }
     else
     {
         /* open the channel */
-        helper->channel = xfconf_channel_new ("pointers");
+        helper->channel = xfconf_channel_get ("pointers");
 
         /* restore the pointer devices */
         xfce_pointers_helper_restore_devices (helper, NULL);
@@ -162,7 +164,7 @@ xfce_pointers_helper_init (XfcePointersHelper *helper)
         gdk_flush ();
         gdk_error_trap_push ();
 
-        
+
         if (G_LIKELY (xdisplay))
         {
             /* monitor device changes */
@@ -182,20 +184,6 @@ xfce_pointers_helper_init (XfcePointersHelper *helper)
 
 
 
-static void
-xfce_pointers_helper_finalize (GObject *object)
-{
-    XfcePointersHelper *helper = XFCE_POINTERS_HELPER (object);
-
-    /* release the channel */
-    if (G_LIKELY (helper->channel))
-        g_object_unref (G_OBJECT (helper->channel));
-
-    (*G_OBJECT_CLASS (xfce_pointers_helper_parent_class)->finalize) (object);
-}
-
-
-
 static void
 xfce_pointers_helper_change_button_mapping_swap (guchar   *buttonmap,
                                                  gshort    num_buttons,
@@ -270,15 +258,21 @@ xfce_pointers_helper_change_button_mapping (XDeviceInfo *device_info,
         buttonmap = g_new0 (guchar, num_buttons);
 
         /* get the button mapping */
+        gdk_error_trap_push ();
         XGetDeviceButtonMapping (xdisplay, device, buttonmap, num_buttons);
+        if (gdk_error_trap_pop ())
+        {
+            g_warning ("Failed to get button mapping");
+            goto out;
+        }
 
         if (right_handed != -1)
         {
             /* get the right button number */
-            right_button = num_buttons < 3 ? 2 : 3;
+            right_button = MIN (num_buttons, 3);
 
             /* check the buttons and swap them if needed */
-            xfce_pointers_helper_change_button_mapping_swap (buttonmap, num_buttons, 1, right_button, !!right_handed);
+            xfce_pointers_helper_change_button_mapping_swap (buttonmap, num_buttons, 1, right_button, right_handed);
         }
 
         if (reverse_scrolling != -1 && num_buttons >= 5)
@@ -288,10 +282,13 @@ xfce_pointers_helper_change_button_mapping (XDeviceInfo *device_info,
         }
 
         /* set the new button mapping */
+        gdk_error_trap_push ();
         XSetDeviceButtonMapping (xdisplay, device, buttonmap, num_buttons);
+        if (gdk_error_trap_pop ())
+          g_warning ("Failed to set button mapping");
 
         /* cleanup */
-        g_free (buttonmap);
+out:    g_free (buttonmap);
     }
 }
 
@@ -313,7 +310,7 @@ xfce_pointers_helper_change_feedback (XDevice *device,
                                       gint     threshold,
                                       gdouble  acceleration)
 {
-    XFeedbackState      *states;
+    XFeedbackState      *states, *pt;
     gint                 num_feedbacks;
     XPtrFeedbackControl  feedback;
     gint                 n;
@@ -326,10 +323,10 @@ xfce_pointers_helper_change_feedback (XDevice *device,
     if (G_LIKELY (states))
     {
         /* get the pointer feedback class */
-        for (n = 0; n < num_feedbacks; n++)
+        for (pt = states, n = 0; n < num_feedbacks; n++)
         {
             /* find the pointer feedback class */
-            if (states->class == PtrFeedbackClass)
+            if (pt->class == PtrFeedbackClass)
             {
                 if (acceleration > 0 || acceleration == -1)
                 {
@@ -354,7 +351,7 @@ xfce_pointers_helper_change_feedback (XDevice *device,
                 /* create a new feedback */
                 feedback.class      = PtrFeedbackClass;
                 feedback.length     = sizeof (XPtrFeedbackControl);
-                feedback.id         = states->id;
+                feedback.id         = pt->id;
                 feedback.threshold  = threshold;
                 feedback.accelNum   = num;
                 feedback.accelDenom = denom;
@@ -367,7 +364,7 @@ xfce_pointers_helper_change_feedback (XDevice *device,
             }
 
             /* advance the offset */
-            states = (XFeedbackState *) ((gchar *) states + states->length);
+            pt = (XFeedbackState *) ((gchar *) pt + pt->length);
         }
 
         /* cleanup */
